﻿using System.Globalization;

namespace Magify
{
    public class PurchaseInfo
    {
        public readonly string Price;
        public readonly string Currency;

        /// <summary>
        /// For Android it is orderId from product's receipt
        /// For iOS it is transactionID
        /// </summary>
        public readonly string TransactionId;

        /// <summary>
        /// For Android it is orderId from product's receipt
        /// For iOS it is appleOriginalTransactionID
        /// </summary>
        public readonly string OriginalTransactionId;

        /// <summary>
        /// Represents Android purchase token.
        /// Null for all other platforms.
        /// </summary>
        public readonly string PurchaseToken;

        public readonly PurchaseReceiptModel Receipt;

        public readonly SubscriptionInfo SubscriptionInfo;

        public PurchaseInfo(string price, string currency, string transactionId, string originalTransactionId, string purchaseToken, PurchaseReceiptModel receipt, SubscriptionInfo subscriptionInfo = default)
        {
            Price = price;
            Currency = currency;
            TransactionId = transactionId;
            OriginalTransactionId = originalTransactionId;
            PurchaseToken = purchaseToken;
            Receipt = receipt;
            SubscriptionInfo = subscriptionInfo;
        }

#if UNITY_PURCHASES
        public PurchaseInfo(UnityEngine.Purchasing.Product product, SubscriptionInfo subscriptionInfo = default)
        {
            Price = product.metadata.localizedPrice.ToString(CultureInfo.InvariantCulture);
            Currency = product.metadata.isoCurrencyCode;
            TransactionId = product.GetTransactionID();
            OriginalTransactionId = product.GetOriginalTransactionID();
            PurchaseToken = product.GetPurchaseToken();
            Receipt = new PurchaseReceiptModel(product.receipt);
            SubscriptionInfo = subscriptionInfo;
        }
#endif
    }
}