﻿using System;
using UnityEngine;

namespace Magify
{
    [Serializable]
    public abstract class RemoteProperty<T>
    {
        [SerializeField]
        private string _name;
        [SerializeField]
        private T _defaultValue;

        public T Value
        {
            get
            {
                if (!MagifyManager.Initialized) return _defaultValue;
                return !TryGetValue(_name, out var value) ? _defaultValue : value;
            }
        }

        protected RemoteProperty(string name, T defaultValue)
        {
            _name = name;
            _defaultValue = defaultValue;
        }

        protected abstract bool TryGetValue(string name, out T value);

        public static implicit operator T(RemoteProperty<T> property) => property.Value;
    }

    [Serializable]
    public class NumberRemoteProperty : RemoteProperty<double>
    {
        public NumberRemoteProperty(string name, float defaultValue) : base(name, defaultValue)
        {
        }

        protected override bool TryGetValue(string name, out double value)
        {
            return MagifyManager.Features.TryGetNumber(name, out value);
        }
    }

    [Serializable]
    public class StringRemoteProperty : RemoteProperty<string>
    {
        public StringRemoteProperty(string name, string defaultValue) : base(name, defaultValue)
        {
        }

        protected override bool TryGetValue(string name, out string value)
        {
            return MagifyManager.Features.TryGetString(name, out value);
        }
    }

    [Serializable]
    public class BoolRemoteProperty : RemoteProperty<bool>
    {
        public BoolRemoteProperty(string name, bool defaultValue) : base(name, defaultValue)
        {
        }

        protected override bool TryGetValue(string name, out bool value)
        {
            return MagifyManager.Features.TryGetBool(name, out value);
        }
    }
}